/*
Copyright 2011 MIKAEL GUILLEMOT

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */
package gameEngine;

import generated.ImageLoader;

import java.awt.Color;
import java.awt.DisplayMode;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.RenderingHints;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.image.BufferedImage;
import java.util.ConcurrentModificationException;
import java.util.Vector;

import tools.Tools;

/**
 * moteur bien moche mais bon c'est a faire en 3 jours : objectif : pouvoir
 * afficher 2 monstres, les ui etc. Pour la peine on ve heriter de frame! Et on
 * va aussi en profiter pour recuperer les evenements de la souris. note : on
 * gere un double buffering soft.
 * 
 * Pour eviter les petits pb on gere un joli bouton de fermeture en plus dans
 * cette classe.
 * 
 * @author Mikaël GUILLEMOT
 * @version 1.0 
 * creation : 4 novembre 2011
 * 
 */
public class Engine extends Frame implements MouseListener, MouseMotionListener {
	private static final long serialVersionUID = 1L;
	// c'est ici qu'on set la resolution, c'est la resolution max pour un
	// eeepc 701, et certaines platforme portables!
	public static final int W = 800;
	public static int H = 480;
	private final static int sleepTime = 20;
	private int x0=0,y0=0;

	private BufferedImage close0 = ImageLoader.close0;
	private BufferedImage close1= ImageLoader.close1;
	private BufferedImage close;

	private refresh affichage;
	private Graphics2D buffer;
	private BufferedImage buff;
	private long fps;
	private Vector<Renderable> objectToRender;
	private Vector<MouseEater> selectedObjects;
	private Vector<MoveEater>move;
	private boolean fs = false;
	private boolean autorefresh = true;

	public boolean overload_cursor = false;
	
	public Engine() {
		super("");
		Tools t = Tools.getInstance();
		fs = t.getBooleanFor("FULSCREEN");
		overload_cursor = t.getBooleanFor("REDEFINE_CURSOR");
		objectToRender = new Vector<Renderable>();
		selectedObjects = new Vector<MouseEater>();
		move = new Vector<MoveEater>();
		
		this.setUndecorated(true);
		this.setResizable(false);
		this.addMouseListener(this);
		this.addMouseMotionListener(this);
		
		if(fs) {
			GraphicsDevice myDevice = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice();
			DisplayMode odm = myDevice.getDisplayMode();
			
			DisplayMode[] dms=myDevice.getDisplayModes();
			boolean suported = false;
			for (int i = 0; i < dms.length; i++) {
				if(dms[i].getWidth() == W && dms[i].getHeight()==H)
					suported=true;
			}
			DisplayMode dm;
			if(!suported)H = 600;
				dm = new DisplayMode(W, H, odm.getBitDepth(), odm.getRefreshRate() );
			if(myDevice.isFullScreenSupported()) {		
				myDevice.setFullScreenWindow(this);
			if(myDevice.isDisplayChangeSupported())
				try {
				myDevice.setDisplayMode(dm);
				}catch(Exception e) {
					myDevice.setFullScreenWindow(null);
					H=480;
				}
			else
				myDevice.setFullScreenWindow(null);
			}
		}
		this.setSize(W, H);
		
		buff = new BufferedImage(W, H, BufferedImage.TYPE_4BYTE_ABGR);
		buffer = buff.createGraphics();
		buffer.setRenderingHint(RenderingHints.KEY_RENDERING,
				RenderingHints.VALUE_RENDER_SPEED);
		buffer.setRenderingHint(RenderingHints.KEY_INTERPOLATION,
				RenderingHints.VALUE_INTERPOLATION_BICUBIC);
		buffer.setRenderingHint(RenderingHints.KEY_FRACTIONALMETRICS,
				RenderingHints.VALUE_FRACTIONALMETRICS_ON);
		if (t.getBooleanFor("ANTIALIASING")) {
			buffer.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
					RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
			buffer.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
					RenderingHints.VALUE_ANTIALIAS_ON);
		}

		affichage = new refresh(this);
		this.setLocationRelativeTo(this.getParent());
		this.setBackground(Color.black);
		
		this.setVisible(true);
		fps = 0;
		affichage.start();
	}

	public synchronized void addMouseEater(MouseEater obj) {
		selectedObjects.add(obj);
	}

	public synchronized void removeMouseEater(MouseEater obj) {
		selectedObjects.remove(obj);
	}
	
	public synchronized void addMoveEater(MoveEater obj) {
		move.add(obj);
	}

	public synchronized void removeMoveEater(MoveEater obj) {
		move.remove(obj);
	}

	public synchronized void addRenderListener(Renderable obj) {
		objectToRender.add(obj);
	}

	public synchronized void removeRenderListener(Renderable obj) {
		objectToRender.remove(obj);
	}

	public synchronized void backPaint() {

		repaint();
	}

	@Override
	public void paint(Graphics g) {
		// si on a pas de fond defini on en met un noir:
		buffer.setPaint(Color.black);
		buffer.fillRect(0, 0, W, H-1);
		
		//buffer.drawImage(ImageLoader.cabane0, 0, 0, this);
		try {
			for (Renderable obj : objectToRender)
				obj.render(buffer);
		} catch (Exception e) {
		}
		//on desine la croix de fermeture apres les dessin
		buffer.setPaint(Color.white);
		buffer.drawString("fps : " + fps, 5, 15);
		buffer.drawImage(close, W-16, 0, this);
		//deffin du buffer
		g.drawImage(buff, 0, 0, this);
	}

	/** redefinition de update pour ne pas effacer l'affichage */
	@Override
	public void update(Graphics g) {
		paint(g);
	}

	public void mouseEntered(MouseEvent arg0) {
	}

	public void mouseExited(MouseEvent arg0) {
	}

	public void mousePressed(MouseEvent arg0) {
		for (MouseEater obj : selectedObjects)
			obj.updateMouse(arg0.getX(), arg0.getY(), arg0.getButton(),MouseEater.Pressed);
	}

	public void mouseReleased(MouseEvent arg0) {
		for (MouseEater obj : selectedObjects)
			obj.updateMouse(arg0.getX(), arg0.getY(), arg0.getButton(),MouseEater.Released);
	}

	public void mouseClicked(MouseEvent arg0) {
		if (arg0.getX() > W - 16 && arg0.getY() < 16)
			windowClosing();
		try {
		for (MouseEater obj : selectedObjects)
			obj.updateMouse(arg0.getX(), arg0.getY(), arg0.getButton(),MouseEater.Clicked);
		}
		catch(ConcurrentModificationException e) {
			//Exception a traiter
		}

	}

	public void mouseDragged(MouseEvent arg0) {
	}

	public void mouseMoved(MouseEvent arg0) {
		//curseur
		x0 = arg0.getX();
		y0 = arg0.getY();
		//bouton de fermeture
		if (x0 > W - 16 && y0 < 16)
			close = close1;
		else
			close = close0;
		for (MoveEater obj : move)
			obj.update(arg0.getX(), arg0.getY());
	
		
	}

	public void windowClosing() {
		affichage.arret();
		try {
			affichage.join();
		} catch (InterruptedException e) {
			System.out.println("Erreur terminaison des thread");
			e.printStackTrace();
		}
		buffer.dispose();
		System.out.println("Fin du processus d'affichage");
		System.exit(0);
	}

	private class refresh extends Thread {
		private Engine mere;
		private boolean run = true;
		long time, t0 = 0;
		int fc;

		public refresh(Engine m) {
			super();
			mere = m;
			fc = 0;
			t0 = System.nanoTime() / 1000000;
		}

		@Override
		public void run() {
			while (run == true) {
				// ~55x/s
				try {
					Thread.sleep(sleepTime);
				} catch (InterruptedException e) {
				}
				if (autorefresh) {
					mere.backPaint();
					fc++;
					if (fc == 10) {
						time = System.nanoTime() / 1000000;
						fps = fc * 1000 / (time - t0);
						fc = 0;
						t0 = time;
					}
				}
			}
		}

		public void arret() {
			run = false;
		}
	}

}
