/*
Copyright 2011 MIKAEL GUILLEMOT

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */
package gameObjects;

import gameEngine.Activable;
import gameEngine.Engine;
import gameEngine.Renderable;
import generated.ImageLoader;

import java.awt.Color;
import java.awt.Graphics2D;

import ui.UIFight;

/**
 * ici on gere les barres d'attaque
 * 
 * @author Mikael GUILLEMOT
 * @version 1.0
 * creation :  6 novembre 2011
 * 
 */
public class CombatBar implements Renderable, Activable {

	private final Color border = new Color(150, 150, 150);
	

	private Minion minion;
	private int index;
	private int x0 = 0;
	private int x1 = 0;
	private int y0 = 0;
	
	private int astat = 0;
	
	private Button attaque = null;
	private UIFight f;
	


	public CombatBar(int x, int y, UIFight fi, Minion m, int statindex) {
		f=fi;
		astat = m.readStatValue(statindex);
		minion= m;
		index = statindex;
		x0=x;
		y0=y;
		x1 = x0+310; //taille de la zone sensible au passage de la souris

		//on ajoute les boutons
		if(astat != 0)
			attaque = new Button(x0+210, y0+2, ImageLoader.bt0, ImageLoader.bt1,
				0,"       Utiliser");

		
	}
	/**
	 * permet d'enregistrer le composant aupres du moteur
	 */
	public void engineRegister(Engine e) {
		if(attaque !=null) {
			attaque.setActiveListener(this);	
			e.addMouseEater(attaque);	
			e.addRenderListener(attaque);	
		}
		e.addRenderListener(this);
	}
	/**
	 * permet de liberer le moteur
	 */
	public void engineClean(Engine e) {
		if(attaque !=null) {
		e.removeRenderListener(attaque);
		attaque.removeActiveListener();
		e.removeMouseEater(attaque);
		}
		e.removeRenderListener(this);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see gameEngine.Renderable#render(java.awt.Graphics2D)
	 */
	@Override
	public void render(Graphics2D buffer) {
		// text
		buffer.setPaint(Color.darkGray);
		buffer.drawLine(x0, y0+25, x1, y0+25);
		buffer.setPaint(border);
		buffer.drawString(minion.readStatName(index), x0+20, y0+15);
		buffer.drawString("/", x0+170, y0+15);
		buffer.setPaint(Color.red);
		buffer.drawString(Integer.toString(minion.readStatValue(index)), x0+150, y0+15);
		buffer.setPaint(Color.green);
		buffer.drawString(Integer.toString(minion.readStatValue(Minion.DEF_OFFSET+index)), x0+180, y0+15);
		
	}

	/* (non-Javadoc)
	 * @see gameEngine.Activable#activate(int)
	 */
	@Override
	public void activate(int id) {
		f.activate(index);
		
	}
	
}
