/*
Copyright 2011 MIKAEL GUILLEMOT

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */
package gameObjects;

import gameEngine.Renderable;
import generated.ImageLoader;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;

import tools.Tools;

/**
 * Ceci est le monstre, il regroupe toutes les données c'est pas efficace ya des
 * public partout c'est plus simple.
 * 
 * @author Mikael GUILLEMOT
 * @version 1.0 creation : 4 novembre 2011
 * 
 */
public class Minion implements Renderable{

	public static final int CINETIC = 0;
	public static final int ENERGY = 1;
	public static final int RAY = 2;
	public static final int CHIMICAL = 3;
	public static final int PSY = 4;
	public static final int ELEC = 5;

	public static final int CINETIC_RES = 6;
	public static final int ENERGY_RES = 7;
	public static final int RAY_RES = 8;
	public static final int CHIMICAL_RES = 9;
	public static final int PSY_RES = 10;
	public static final int ELEC_RES = 11;

	// valeur de resolution de combat et type devenements
	public static final int NORMAL = 1;
	public static final int DODGE = 2;
	public static final int RESIST = 3;
	public static final int CRITICAL = 4;
	public static final int BACKSTAB = 5;

	// type de minion (gauche droite)
	public static final int GAUCHE = 1;
	public static final int DROITE = 2;
	
	
	private final Color dred = new Color(100,0,0);
	private final Color dorange = new Color(100,50,0);
	private final Color dgreen = new Color(0,100,10);

	private final String[] statsname = { "Attaque cinétique",
			"Attaque énergétique", "Attaque rayonnement", "Attaque chimique",
			"Attaque psychique", "Attaque électrique",

			"Protection cinétique", "Protection énergétique",
			"Protection rayonnement", "Protection chimique",
			"Protection psychique", "Protection électrique" };

	private final String[] statsdesc = {
			"Comme tout les monstre votre serviteur à des griffes acérés... mais sais t'il s'en servir ?",
			"Pour un bon serviteur démoniaque, cracher du feu est un jeu d'enfant. #A défaut de dont, une bombe aérosol, un briquet, et le tour est joué.",
			"Certains minions vicieux tirent des lasers avec leurs yeux, très utile pour les partys disco mais attention a la réflexion. #Cette attaque peu percer les armures, mais est soumise a un seuil de dégâts.",
			"Un vilain crachat acide, très utile pour dissoudre plein de chose: nourriture, amis, ennemi, protections gênante,... #Par contre cela peu prendre du temps. ",
			"Quel monstre m'a jamais rêvé de faire exploser des \"choses\" par la force de sa pensé? #Le problème c'est que ce n'est pas simple si simple...",
			"Grace a vos études approfondie sur les poissons électriques, et vos travaux en géni génétique, #c'est le \"coup de foudre\" qui guette votre adversaire",

			"L’armure de base protège contre les attaques cinétiques mais augmente la durée des dégâts énergétique.",
			"Pour ne pas bruler rien de tel qu'une carapace d'amiante matelassée, en plus c’est plus confortable !",
			"Un peu de polish sur une belle armure d'acier, ca fait peut être pas peur dans les entrailles de l’enfer. #Mais attention au retour de rayon!",
			"Une belle cape en peau de gargouille devrais faire l’affaire pour protéger vos abomination contre les fluides ennemi.",
			"Pour se proteger contre les attaques psychique il n'y a qu'un seul moyen : comprendre ces attaques! #Plus votre serviteur est entrainé a effectuer ce type d'attaque plus il y sera resistant.",
			"Grace a une mise a la terre et un astucieux tissage de cuivre venant renforcer vos protections, #vos monstres pourrons affronter la foudre sans être carbonisé !", };

	private final int poolsize = Tools.getInstance().getIntFor("INIT_STAT");
	private final int statcap = 10;
	public static final int STAT_COUNT = 12;
	public static final int DEF_OFFSET = 6;

	public String name;
	public BufferedImage picture;
	public int imgindex = 0;

	private int[] stats = new int[STAT_COUNT];
	private int critical = 0;
	private int esquive = 0;
	private int life = 100;
	private boolean dead = false;
	private int miniontype = 0;
	static private int damageMult = Tools.getInstance().getIntFor("DMG_MULTI");
	
	public boolean ready = true;
	private int eventlag = 0;
	private BufferedImage eventPic = null;
	private String event = null;
	private int xe = 0;

	/*
	 * chaque point mis en def augmente l'esquive chaque point mis en attaque
	 * augmente les chance de critique
	 * 
	 * TODO ajouter les capacitées special (et voir leur cout)
	 * et les correlations
	 */

	private int pool = poolsize;

	public Minion(String n, int imgindex, int cote) {
		name = n;
		miniontype = cote;
		this.imgindex = imgindex;
		if (miniontype == GAUCHE)
			picture = PictureGallery.getG(imgindex);
		else
			picture = PictureGallery.getD(imgindex);
		for (int i = 0; i < stats.length; i++) {
			stats[i] = 0;
		}
	}

	public int getPool() {
		return pool;
	}

	public void nextPic() {
		imgindex = PictureGallery.next(imgindex);
		if (miniontype == GAUCHE)
			picture = PictureGallery.getG(imgindex);
		else
			picture = PictureGallery.getD(imgindex);
	}

	public void prevPic() {
		imgindex = PictureGallery.prev(imgindex);
		if (miniontype == GAUCHE)
			picture = PictureGallery.getG(imgindex);
		else
			picture = PictureGallery.getD(imgindex);
	}

	public int getCritical() {
		int sum = 0;
		for (int i = 0; i < DEF_OFFSET; i++) {
			sum += stats[i];
		}
		critical = sum * 6 / DEF_OFFSET;
		return critical;
	}

	public int getEsquive() {
		int sum = 0;
		for (int i = DEF_OFFSET; i < STAT_COUNT; i++) {
			sum += stats[i];
		}
		esquive = sum * 6 / DEF_OFFSET;
		return esquive;
	}

	public String readStatName(int stat) {
		return statsname[stat];
	}

	public String readStatDescription(int stat) {
		return statsdesc[stat];
	}

	public int readStatValue(int stat) {
		return stats[stat];
	}

	public boolean incStat(int statindex) {
		if (statindex == Minion.PSY || statindex == Minion.PSY_RES) {
			// car particulier des stat liées
			if (pool > 1 && stats[Minion.PSY] < statcap
					&& stats[Minion.PSY_RES] < statcap) {
				stats[Minion.PSY]++;
				pool--;
				stats[Minion.PSY_RES]++;
				pool--;
				return true;
			}
		} else if (pool > 0 && stats[statindex] < statcap) {
			stats[statindex]++;
			pool--;
			return true;
		}
		return false;
	}

	public boolean decStat(int statindex) {
		if (statindex == Minion.PSY || statindex == Minion.PSY_RES) {
			// car particulier des stat liées
			if (stats[Minion.PSY] > 0 && stats[Minion.PSY_RES] > 0) {
				stats[Minion.PSY]--;
				pool++;
				stats[Minion.PSY_RES]--;
				pool++;
				return true;
			}
		} else if (stats[statindex] > 0) {
			stats[statindex]--;
			pool++;
			return true;
		}
		return false;
	}

	public boolean checkStat() {
		int sum = 0;
		for (int i = 0; i <DEF_OFFSET ; i++) {
			sum += stats[i];
		}
		if (sum >0)
			return true;
		return false;
	}

	public void takeDamage(int value) {
		life = life - value;
		if (life <= 0) {
			dead = true;
		}

	}

	public boolean isdead() {
		return dead;
	}

	/**
	 * 
	 * @param attaque
	 * @param defense
	 * @param statAttaqueIndex
	 */
	public static void CombatSolver(Minion att, Minion def, int statAttaqueIndex) {
		if (Math.random() * 100 < def.getEsquive()) {
			def.addEvent(DODGE,0);
			return;
			}

		int result = NORMAL;
		int damage = new Double(Math.random() * 4).intValue();
		int defVal = def.readStatValue(statAttaqueIndex + DEF_OFFSET);
		if (Math.random() * 100 < att.getCritical()) {
			result = CRITICAL;
			defVal = 0;
			damage += new Double(Math.random() * 4).intValue();
		}
		damage += att.readStatValue(statAttaqueIndex);
		damage-= defVal;
		damage *= damageMult;
		damage /=10;
		if (damage > 0)
			def.takeDamage(damage);
		else
			result =  RESIST;
		def.addEvent(result,damage);
	}

	/**
	 * 
	 * @param attaque
	 * @param defense
	 */
	public static void backstab(Minion att, Minion def) {
		if (Math.random() * 100 < def.getEsquive())
			def.addEvent(DODGE,0);
		else {
			int damage = 2+new Double(Math.random() * 10).intValue();
			def.takeDamage(damage);
			def.addEvent( BACKSTAB,damage);
		}	
	}
	
	//FIXME pour debug
	public void autoset(int set) {
		switch (set) {
		case 1:
			pool= 0;
			stats[CINETIC] = 10;
			stats[RAY] = 10;
			stats[CHIMICAL_RES] = 10;
			stats[ENERGY_RES] = 10;
			stats[RAY_RES] = 10;
			break;
		case 2:
			pool= 0;
			stats[CHIMICAL] = 10;
			stats[ENERGY] = 10;
			stats[CINETIC_RES] = 10;
			stats[ENERGY_RES] = 10;
			stats[RAY_RES] = 10;
			break;
		case 3:
			pool= 0;
			stats[PSY] = 10;
			stats[ENERGY] = 10;
			stats[CINETIC_RES] = 10;
			stats[PSY_RES] = 10;
			stats[RAY_RES] = 10;
			break;

		default:
			break;
		}

	}

	/* (non-Javadoc)
	 * @see gameEngine.Renderable#render(java.awt.Graphics2D)
	 */
	@Override
	public void render(Graphics2D buffer) {
		int x,y = 100;
		if(miniontype == DROITE)
			x=550;
		else
			x=100;
			buffer.drawImage(picture, x, y+20, null);
			x=x+10;
			if(life >70) {
				buffer.setPaint(dgreen);
				buffer.fillRect(x, y, 100, 10);
				buffer.setPaint(Color.green);
				buffer.fillRect(x, y, life, 10);
			}else if(life >35) {
				buffer.setPaint(dorange);
				buffer.fillRect(x, y, 100, 10);
				buffer.setPaint(Color.orange);
				buffer.fillRect(x, y, life, 10);
			}
			else
			{
				buffer.setPaint(dred);
				buffer.fillRect(x, y, 100, 10);
				buffer.setPaint(Color.red);
				buffer.fillRect(x, y, life, 10);
			}
			buffer.setPaint(Color.darkGray);
			buffer.drawRect(x, y, 100, 10);
			
			//dessin de evenement
			if(eventlag>0)
			{
				if(eventPic!=null && eventlag>50)
					buffer.drawImage(eventPic, xe, 150, null);
				if(event!=null) {
					buffer.setPaint(Color.white);
					buffer.drawString(event, x, eventlag);
				}
				eventlag-=2;
				if(eventlag<=0)
				{
					eventPic=null;
					event=null;
					ready = true;
				}
			}
				
		
	}
	
	/**
	 * gestion de evenement de jeu
	 * @param type
	 * @param value
	 */
	private void addEvent(int type,int value) {
		eventlag=100;
		ready = false;
		if(miniontype == DROITE)
			xe=550;
		else
			xe=100;
		switch (type) {
		case BACKSTAB:
			xe=200;
			eventPic=ImageLoader.backstab;
			event="-"+value;
			break;
		case NORMAL:
			eventPic=null;
			event="-"+value;
			break;
		case DODGE:
			eventPic=ImageLoader.dodge;
			event=null;
			break;
		case CRITICAL:
			eventPic=ImageLoader.critical;
			event="-"+value;
			break;
		case RESIST:
			eventPic=null;
			event= "0 (resisté!)";
			break;
		default:
			break;
		}
	}
	
	/**
	 * attend la fin de l'anim
	 */
	public synchronized void waitAnim() {
		while(!ready) {
			try {
				Thread.sleep(100);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
	}
	
}
