/*
Copyright 2011 MIKAEL GUILLEMOT

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */
package gameObjects;

import gameEngine.Activable;
import gameEngine.Engine;
import gameEngine.MoveEater;
import gameEngine.Renderable;
import generated.ImageLoader;

import java.awt.Color;
import java.awt.Graphics2D;

/**
 * ici on gere la selection des stats
 * 
 * @author Mikael GUILLEMOT
 * @version 1.0
 * creation :  5 novembre 2011
 * 
 */
public class StatBar implements Renderable, MoveEater, Activable {

	private final Color border = new Color(150, 150, 150);
	private final Color ibback = new Color(100,75,0,80);
	private final int IDPrev = 0;
	private final int IDNext = 1;
	

	private Minion minion;
	private int index;
	private int x0 = 0;
	private int x1 = 0;
	private int y0 = 0;
	private int y1 = 0;
	
	private String label;
	private String description[];
	private boolean dispinf = false;
	private GButton prev;
	private GButton next;

	public StatBar(int x, int y, Minion m, int statindex) {
		minion= m;
		index = statindex;
		this.label = minion.readStatName(index);
		this.description = minion.readStatDescription(index).split("#");
		x0=x;
		y0=y;
		x1 = x0+460; //taille de la zone sensible au passage de la souris
		y1 = y0+20;
		//on ajoute les boutons
		prev = new GButton(x0+50, y0+2, ImageLoader.btmoins0, ImageLoader.btmoins1,
				IDPrev);
		next = new GButton(x0+140, y0+2, ImageLoader.btplus0, ImageLoader.btplus1,
				IDNext);
		
	}

	// ENERGIE : + 10 + (hover + image)

	/**
	 * permet d'enregistrer le composant aupres du moteur
	 */
	public void engineRegister(Engine e) {
		prev.setActiveListener(this);
		next.setActiveListener(this);
		
		e.addMouseEater(prev);
		e.addMouseEater(next);
		
		e.addMoveEater(this);
		
		e.addRenderListener(this);
		e.addRenderListener(prev);
		e.addRenderListener(next);
	}
	/**
	 * permet de liberer le moteur
	 */
	public void engineClean(Engine e) {
		e.removeRenderListener(prev);
		e.removeRenderListener(next);
		e.removeRenderListener(this);
		
		e.removeMouseEater(prev);
		e.removeMouseEater(next);
		
		prev.removeActiveListener();
		next.removeActiveListener();
		
		e.removeMoveEater(this);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see gameEngine.MoveEater#updateMouse(int, int)
	 */
	@Override
	public void update(int x, int y) {
		if (x < x1 && x > x0 && y < y1 && y > y0) {
			dispinf = true;
		} else
			dispinf = false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see gameEngine.Renderable#render(java.awt.Graphics2D)
	 */
	@Override
	public void render(Graphics2D buffer) {
		// infobule
		if (dispinf) {
			buffer.setPaint(ibback);
			buffer.fillRect(30, 405, 740, 60);
			buffer.fillRect(x0, y0, x1-x0, 20);
			buffer.setPaint(border);
			buffer.drawRect(30, 405, 740, 60);
			int yof = 405+15;
			for (int i = 0; i < description.length; i++) {
				buffer.drawString(description[i],50, yof);
				yof+=20;
			}
		}
		// text
		buffer.setPaint(border);
		buffer.drawLine(x0, y0+20, x1, y0+20);
		buffer.drawString(label, x0+190, y0+15);
		buffer.setPaint(Color.white);
		buffer.drawString(Integer.toString(minion.readStatValue(index)), x0+100, y0+15);
		//TODOafficher les dessin des stat ... un jour

	}

	/* (non-Javadoc)
	 * @see gameEngine.Activable#activate(int)
	 */
	@Override
	public void activate(int id) {
		switch (id) {
		case IDPrev:
			minion.decStat(index);
			break;
		case IDNext:
			minion.incStat(index);
			break;
		default:
			break;
		}
		
	}
}
