/*
  *This program is free software. It comes without any warranty, to
  * the extent permitted by applicable law. You can redistribute it
  * and/or modify it under the terms of the Do What The Fuck You Want
  * To Public License, Version 2, as published by Sam Hocevar. See
  * http://sam.zoy.org/wtfpl/COPYING for more details. 
 */
package tools;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * on cette classe et autonome (avec un main). Elle permet de generer le code
 * des classes générées Les fichiers images doivent etre supporté par java (png,
 * jpeg, ...) et leur nom ne doit pas contenir d'espace, de points, de
 * caracteres spéciaux en general.
 * 
 * @author Mikael GUILLEMOT
 * @version 1.0 creation : 4 novembre 2011
 * 
 */
public class PictureLoaderGenerator {

	private static Tools t;

	public static void main(String[] args) throws IOException,
			InterruptedException {
		Tools.PropertiesGenerator pg = new MAConfigGenerator();
		Tools._setPropertiesFile("./data/MinionsArena.properties");
		Tools._setPropertiesGenerator(pg);
		t = Tools.getInstance();

		buildImageLoaderSrc("src/generated/");
	}

	/**
	 * generation de ImageLoader.java, attention si le fichier existe il est
	 * ecrasé!
	 * 
	 * @param path
	 *            ou mettre la sourtie (code source)
	 * @throws IOException
	 * @throws InterruptedException
	 */
	private static void buildImageLoaderSrc(String path) throws IOException,
			InterruptedException {
		// initialisation
		final String header = "/*\n#COPYRIGHT_NOTICE\n\n*/package generated;\n\nimport java.awt.image.BufferedImage;\nimport java.io.File;\nimport javax.imageio.ImageIO;\nimport tools.Tools;\n\n"
				+ "/**\n * WARNING : This file is generated automatically.\n * Do not edit manually!\n * Ask the author for modifications if needed.\n * \n * Created on: "
				+ new SimpleDateFormat("dd MMM yyyy").format(new Date())
				+ " \n * @author: Mikael Guillemot\n */\n\n"
				+ "public class ImageLoader {\n";
		final String code1 = "public static String Start() {String path = Tools.getInstance().getStringFor(\"PICTURE_PATH\");File f=null;try {\n";
		final String code2 = "} catch (Exception e1) {return f.getAbsolutePath();}return null;}}";

		// preparation des données
		t.log("generation de ImageLoader.java");
		List<File> piclist = new ArrayList<File>();
		t.getFilesIn(t.getStringFor("PICTURE_PATH"), piclist);

		for (File pic : piclist) {
			// on fait un test rapide pour les espaces dans le nom de fichier
			// (non supporté)
			if (pic.getName().contains(" "))
				t.logError("nom invalide (pas d'espace dans le nom : "
						+ pic.getName());
		}

		File sourceFile = new File(path + "ImageLoader.java");
		if (sourceFile.exists())
			sourceFile.delete();

		RandomAccessFile raf = new RandomAccessFile(sourceFile, "rw");
		String subname = null;
		// génération
		raf.writeBytes(header);
		for (File pic : piclist) {
			// Ici c'est un peu bourrin, on considere que le fichier n'a pas de
			// . dans son nom. En effet en java les variable ne supertent pas
			// ces caracteres
			subname = pic.getName().substring(0, pic.getName().indexOf('.'));
			raf.writeBytes("public static BufferedImage " + subname + ";\n");
		}
		raf.writeBytes("\n");
		raf.writeBytes(code1);
		for (File pic : piclist) {
			t.log("traitement de " + pic.getName());
			subname = pic.getName().substring(0, pic.getName().indexOf('.'));
			raf.writeBytes("f = new File(path + \"" + pic.getName() + "\");\n");
			raf.writeBytes(subname + " = ImageIO.read(f);\n");
		}

		raf.writeBytes(code2);
		raf.writeBytes("\n");
		raf.close();
		t.log("terminé");
	}
}
