/*
Copyright 2011 MIKAEL GUILLEMOT

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */
package ui;

import gameEngine.Activable;
import gameEngine.Engine;
import gameEngine.Renderable;
import gameObjects.CombatBar;
import gameObjects.FTabs;
import gameObjects.Minion;
import generated.ImageLoader;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Toolkit;

/**
 * 
 * classe UI pour les combat
 * 
 * @author Mikael GUILLEMOT
 * @version 1.0 creation : 5 novembre 2011
 * 
 */
public class UIFight implements Renderable, Activable {

	private boolean isactive = true;
	private boolean victory = false;
	private Minion[] iam;
	private Minion[] usrm;
	private int iami = 0;
	private int usrmi = 0;
	private Engine en;
	private long imagecounter = 0;
	private int feu = 0;
	private int ombre = 0;
	private int sensfeu = 1;
	private int sensombre = 1;
	private FTabs tab;
	private CombatBar[] barres;

	public UIFight(Engine e, Minion[] userminion, Minion[] iaminion) {
		en = e;
		iam = iaminion;
		usrm = userminion;
		barres = new CombatBar[Minion.DEF_OFFSET];
		e.addRenderListener(this);
		Toolkit tk = Toolkit.getDefaultToolkit();
		if (e.overload_cursor) {
			Cursor cur = tk.createCustomCursor(ImageLoader.cursor, new Point(0,
					0), "hcur");
			e.setCursor(cur);
		}

		addMinion(iam[iami]);
		addMinion(usrm[usrmi]);
		tab = new FTabs(usrm);

		tab.setActiveListener(this);
		e.addMouseEater(tab);
		e.addRenderListener(tab);
		bindAttakpage();
		while (isactive)
			try {
				// ici on teste les condition d'arret
				if (iam[0].isdead() && iam[1].isdead() && iam[2].isdead()) {
					System.out.println("fin de partie!");
					victory = true;
					isactive = false;
				}
				if (usrm[0].isdead() && usrm[1].isdead() && usrm[2].isdead()) {
					isactive = false;
				}

				Thread.sleep(100);
			} catch (InterruptedException e1) {
				e1.printStackTrace();
			}
		unbindAttakpage();
		tab.removeActiveListener();
		e.removeMouseEater(tab);
		e.removeRenderListener(tab);

		removeMinion(iam[iami]);
		removeMinion(usrm[usrmi]);
		e.removeRenderListener(this);
		e = null;
	}

	public boolean isVictorious() {
		return victory;
	}

	private void changeMinion(Minion[] m, int old, int neww) {
		removeMinion(m[old]);
		addMinion(m[neww]);

	}

	/* ajoute un minion sur la table de jeu */
	private void addMinion(Minion m) {
		en.addRenderListener(m);
	}

	private void removeMinion(Minion m) {
		en.removeRenderListener(m);

	}

	private void bindAttakpage() {

		for (int i = 0; i < barres.length; i++) {
			barres[i] = new CombatBar(40, 300 + i * 26, this, usrm[usrmi], i);
			barres[i].engineRegister(en);
		}

	}

	private void unbindAttakpage() {
		for (int i = 0; i < barres.length; i++) {
			barres[i].engineClean(en);
			barres[i] = null;
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see gameEngine.Renderable#render(java.awt.Graphics2D)
	 */
	@Override
	public void render(Graphics2D buffer) {
		// affichage du feu
		imagecounter++;
		buffer.drawImage(ImageLoader.fire0, 0, Engine.H - 236 + feu, null);
		buffer.drawImage(ImageLoader.fire1, 0, Engine.H - 236 - feu, null);
		if (imagecounter % 20 == 0)
			sensfeu = sensfeu * -1;
		feu += sensfeu;
		if (imagecounter % 50 == 0)
			sensombre = sensombre * -1;
		ombre += sensombre;
		// estomper feu
		buffer.setPaint(new Color(0, 0, 0, 100 - (ombre)));
		buffer.fillRect(0, 0, Engine.W, Engine.H);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see gameEngine.Activable#activate(int)
	 */
	@Override
	public void activate(int id) {
		// methode de jeu
		if (id == FTabs.id) {
			new BSActioner().start();
		} else if (usrm[0].ready && usrm[1].ready && usrm[2].ready
				&& iam[0].ready && iam[1].ready && iam[2].ready) {
			if (!usrm[usrmi].isdead())
				new Attack(id).start();
		}

	}

	private class Attack extends Thread {
		int id = 0;

		Attack(int i) {
			id = i;
		}

		@Override
		public void run() {
				//on gere l'attaque
				Minion.CombatSolver(usrm[usrmi], iam[iami], id);
				iam[iami].waitAnim();
				//si l'odi meur, il change
				if(iam[iami].isdead()) {
					if(iami==2)return;
					changeMinion(iam, iami, iami+1);
				iami++;
				}
				//l'ordi attaque
				int bi =0, max = 0, tmp;
				for (int i = 0; i < Minion.DEF_OFFSET; i++) {
					tmp =iam[iami].readStatValue(i)-usrm[usrmi].readStatValue(i+Minion.DEF_OFFSET);
					if (tmp>max)
						{bi=i;
						max = tmp;}
				}
				//System.out.println("choix d'attaque "+iam[iami].readStatName(bi)+iam[iami].readStatValue(bi)+" contre "+usrm[usrmi].readStatValue(bi+Minion.DEF_OFFSET));
				Minion.CombatSolver(iam[iami], usrm[usrmi], bi);
				iam[iami].waitAnim();
		}
	}

	private class BSActioner extends Thread {

		@Override
		public void run() {
			// on change de minion : on se prend une attaque
			if (!usrm[usrmi].isdead()) {
				Minion.backstab(iam[iami], usrm[usrmi]);
				usrm[usrmi].waitAnim();
			}
			// gestion du changement de tabulation
			// is se passe un tour
			unbindAttakpage();
			changeMinion(usrm, usrmi, tab.selected);
			usrmi = tab.selected;
			bindAttakpage();
		}
	}

}
