/*
Copyright 2011 MIKAEL GUILLEMOT

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */
package ui;

import gameEngine.Activable;
import gameEngine.Engine;
import gameEngine.Renderable;
import gameObjects.Button;
import gameObjects.GButton;
import gameObjects.Minion;
import gameObjects.StatBar;
import gameObjects.Tabs;
import generated.ImageLoader;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Toolkit;

import tools.Names;

/**
 * 
 * UI de base, pour la premiere etape : selection des personnages
 * 
 * @author Mikael GUILLEMOT
 * @version 1.0 creation : 5 novembre 2011
 * 
 */
public class UIStats implements Renderable, Activable {

	private final int IDFight = 2;
	private final int IDPicNext = 3;
	private final int IDPicPrv = 4;
	private final int IDRandName = 5;

	private Minion[] minions;
	private int index = 0;
	private boolean isactive = true;
	private Tabs top;
	private Engine en;
	private StatBar[] stats;

	public UIStats(Engine e, Minion[] m) {

		minions = m;
		en = e;
		Toolkit tk = Toolkit.getDefaultToolkit();
		if (e.overload_cursor) {
			Cursor cur = tk.createCustomCursor(ImageLoader.cursor, new Point(0,
					0), "cur");
			e.setCursor(cur);
		}
		stats = new StatBar[Minion.STAT_COUNT];
		top = new Tabs(minions);

		// ajout des elements
		Button next = new Button(660, 10, ImageLoader.bt0, ImageLoader.bt1,
				IDFight, "   Attaquer ! ->");
		Button rann = new Button(105, 300, ImageLoader.bt0, ImageLoader.bt1,
				IDRandName, "  Nom aléatoire");
		GButton picprv = new GButton(50, 180, ImageLoader.btmoins0,
				ImageLoader.btmoins1, IDPicPrv);
		GButton picnxt = new GButton(230, 180, ImageLoader.btplus0,
				ImageLoader.btplus1, IDPicNext);

		// liste des objets que l'on ecoute
		top.setActiveListener(this);
		next.setActiveListener(this);
		picprv.setActiveListener(this);
		picnxt.setActiveListener(this);
		rann.setActiveListener(this);

		// liste des objet a l'ecoute de la souris
		e.addMouseEater(top);
		e.addMouseEater(next);
		e.addMouseEater(picprv);
		e.addMouseEater(picnxt);
		e.addMouseEater(rann);

		// liste des objets a l'ecoute des monvements
		e.addMoveEater(top);

		// liste des objets a dessiner
		e.addRenderListener(this);
		e.addRenderListener(top);
		e.addRenderListener(next);
		e.addRenderListener(picprv);
		e.addRenderListener(picnxt);
		e.addRenderListener(rann);

		// ici on ajoute la page d'index 1, par la suite , au changement de tab
		// on doit la rafraichir.
		bindStatpage();

		while (isactive)
			try {
				Thread.sleep(100);
			} catch (InterruptedException e1) {
				e1.printStackTrace();
			}

		unbindStatpage();
		
		top.removeActiveListener();
		next.removeActiveListener();
		picprv.removeActiveListener();
		picnxt.removeActiveListener();
		rann.removeActiveListener();

		e.removeMouseEater(top);
		e.removeMouseEater(next);
		e.removeMouseEater(picprv);
		e.removeMouseEater(picnxt);
		e.removeMouseEater(rann);

		e.removeMoveEater(top);

		e.removeRenderListener(top);
		e.removeRenderListener(next);
		e.removeRenderListener(picprv);
		e.removeRenderListener(picnxt);
		e.removeRenderListener(rann);
		e.removeRenderListener(this);

	}

	private void bindStatpage() {
		int statseparator=0;
		for (int i = 0; i < stats.length; i++) {
			stats[i] = new StatBar(320, 64 + i * 26+statseparator, minions[index], i);
			stats[i].engineRegister(en);
			if(i==5)
				statseparator=10;
		}

	}

	private void unbindStatpage() {
		for (int i = 0; i < stats.length; i++) {
			stats[i].engineClean(en);
			stats[i] = null;
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see gameEngine.Activable#activate(int)
	 */
	@Override
	public void activate(int id) {
		if (id == Tabs.id) {
			// gestion du changement de tabulation
			unbindStatpage();
			index = top.selected;
			bindStatpage();
		}
		if (id == IDRandName) {
			minions[index].name = Names.random();
			top.updateNamesPosition();
		}
		if (id == IDPicNext) {
			minions[index].nextPic();
		}
		if (id == IDPicPrv) {
			minions[index].prevPic();
		}
		if (id == IDFight) {
			// TODO Validation de la repartition pour tout les minions
			if(minions[0].checkStat() && minions[1].checkStat() && minions[2].checkStat() )
			isactive = false;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see gameEngine.Renderable#render(java.awt.Graphics2D)
	 */
	@Override
	public void render(Graphics2D buffer) {

		buffer.drawImage(ImageLoader.cabane0, 0, 0, null);
		// end debug
		buffer.setPaint(Color.white);
		buffer.drawLine(308, 64, 308, 400);
		buffer.setPaint(Color.gray);
		buffer.drawLine(309, 65, 309, 400);
		// buffer.drawLine(50, 400, 750, 400);
		buffer.setPaint(Color.black);
		buffer.fillRect(88, 118, minions[index].picture.getWidth() + 4,
				minions[index].picture.getHeight() + 4);
		buffer.setPaint(Color.gray);
		buffer.drawRect(88, 118, minions[index].picture.getWidth() + 4,
				minions[index].picture.getHeight() + 4);
		buffer.drawImage(minions[index].picture, 90, 120, null);
		buffer.setPaint(Color.white);
		buffer.drawString("profile de : " + minions[index].name, 95,
				140 + minions[index].picture.getHeight());
		buffer.drawString("chance d'attaque critique : " + minions[index].getCritical()+"%", 95,360);
		buffer.drawString("chance d'esquive : " + minions[index].getEsquive()+"%", 95,385);
		buffer.drawString(
				"nombre de points restant : "+minions[index].getPool(), 420,
				25);
	}

}
